@@ -1,3 +1,4 @@
+import contextlib
 import logging
 import gi
 
@@ -60,8 +61,8 @@ class Panel(ScreenPanel):
 
     def activate(self):
         self.load_meshes()
-        if not self._printer.get_stat('bed_mesh', 'profile_name') and 'default' in self.profiles:
-            self.send_load_mesh(None, 'default')  # this is not the default behaviour of klipper anymore
+        with contextlib.suppress(KeyError):
+            self.activate_mesh(self._printer.get_stat("bed_mesh", "profile_name"))
 
     def activate_mesh(self, profile):
         if self.active_mesh is not None:
